package org.mmga.clubs.configuration;

import lombok.SneakyThrows;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.interfaces.ECPrivateKey;
import java.security.interfaces.ECPublicKey;
import java.security.spec.ECGenParameterSpec;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;

@Configuration
public class ECKeyProviderConfiguration {
    private KeyPair keyPair;
    private ECPublicKey publicKey;
    private ECPrivateKey privateKey;
    @SneakyThrows
    public KeyPair keyPair(){
        if (this.keyPair != null){
            return this.keyPair;
        }
        KeyPairGenerator keyPairGenerator = KeyPairGenerator.getInstance("EC");
        ECGenParameterSpec ecGenParameterSpec = new ECGenParameterSpec("secp256r1");
        keyPairGenerator.initialize(ecGenParameterSpec);
        this.keyPair = keyPairGenerator.generateKeyPair();
        return this.keyPair;
    }
    @SneakyThrows
    @Bean
    public ECPublicKey ecPublicKey(){
        if (this.publicKey == null){
            File file = new File("jwt_pub.pem");
            if (file.exists()){
                FileInputStream publicKeyInputStream = new FileInputStream(file);
                byte[] publicKeyData = publicKeyInputStream.readAllBytes();
                publicKeyInputStream.close();
                KeyFactory keyFactory = KeyFactory.getInstance("EC");
                this.publicKey = (ECPublicKey) keyFactory.generatePublic(new X509EncodedKeySpec(publicKeyData));
            }else {
                this.publicKey = (ECPublicKey) keyPair().getPublic();
                if (!file.createNewFile()) {
                    throw new RuntimeException("Can't create public key file");
                }
                FileOutputStream fileOutputStream = new FileOutputStream(file);
                fileOutputStream.write(publicKey.getEncoded());
                fileOutputStream.close();
            }
        }
        return this.publicKey;
    }
    @SneakyThrows
    @Bean
    public ECPrivateKey ecPrivateKey(){
        if (this.privateKey == null){
            File file = new File("jwt_pri.pem");
            if (file.exists()){
                FileInputStream publicKeyInputStream = new FileInputStream(file);
                byte[] publicKeyData = publicKeyInputStream.readAllBytes();
                publicKeyInputStream.close();
                KeyFactory keyFactory = KeyFactory.getInstance("EC");
                this.privateKey = (ECPrivateKey) keyFactory.generatePrivate(new PKCS8EncodedKeySpec(publicKeyData));
            }else {
                this.privateKey = (ECPrivateKey) keyPair().getPrivate();
                if (!file.createNewFile()) {
                    throw new RuntimeException("Can't create private key file");
                }
                FileOutputStream fileOutputStream = new FileOutputStream(file);
                fileOutputStream.write(privateKey.getEncoded());
                fileOutputStream.close();
            }
        }
        return this.privateKey;
    }
}
