package org.mmga.clubs.schedule;

import lombok.extern.slf4j.Slf4j;
import org.mmga.clubs.dao.redis.AvatarDao;
import org.mmga.clubs.dao.redis.VerifyDao;
import org.mmga.clubs.entities.user.change.UserAvatarOperationVo;
import org.mmga.clubs.entities.verify.VerifyVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.io.File;
import java.util.Date;

@Slf4j
@Service
public class RemoveTempAvatarSchedule {
    private final AvatarDao avatarDao;
    private final File tempDir;
    private final VerifyDao verifyDao;

    @Autowired
    public RemoveTempAvatarSchedule(AvatarDao avatarDao, File tempDir, VerifyDao verifyDao){
        this.avatarDao = avatarDao;
        this.tempDir = tempDir;
        this.verifyDao = verifyDao;
    }
    @Scheduled(fixedDelay = 1000)
    public void removeTempAvatar() {
        for (UserAvatarOperationVo userAvatarOperationVo : avatarDao.findAll()) {
            long i = new Date().getTime() - userAvatarOperationVo.createTime().getTime();
            if (i >= 300000){
                avatarDao.delete(userAvatarOperationVo);
                String s = userAvatarOperationVo.avatarSHA();
                File file = new File(this.tempDir, s);
                String absolutePath = file.getAbsolutePath();
                log.info("删除了临时头像文件：{}", absolutePath);
                if (file.exists()) {
                    if (!file.delete()) {
                        log.error("无法删除临时头像文件：{}", absolutePath);
                    }
                }
            }
        }
    }
    @Scheduled(fixedDelay = 1000)
    public void removeExpiredVerify() {
        for (VerifyVo verifyVo : verifyDao.findAll()) {
            long i = new Date().getTime() - verifyVo.createTime().getTime();
            if (i >= 300000){
                verifyDao.delete(verifyVo);
            }
        }
    }
}
